function stat = iterative_learning( ...
  D, I_mat, T, S, batch_size, batch_select_method, ...
  proposal_type, aux_param, aux_data)

  [m, n] = size(D);

  % Setup candidate experiments
  C = [enumerate_intervention(n, 0); enumerate_intervention(n, 1)];

  stat = struct;
  stat.runtime = zeros(T + 1, 1);
  stat.expr = cell(T, 1);
  stat.mse = zeros(T + 1, 1);
  stat.auroc = zeros(T + 1, 1);
  stat.auprc = zeros(T + 1, 1);

  tstart = tic;

  G_singleton = struct('n', n, 'adj', sparse(n, n));

  fprintf('Running MCMC on initial data (size = %d) ... \n', m); tic
  G_init = struct('n', n, 'adj', logical(sparse(n,n)));
  [G_list, G_logp, naccept, ntotal] = MH_sample_graph(S, D, I_mat, proposal_type, aux_param.MH_init_burnin, aux_param.MH_init_thinning, G_init);
  fprintf('done.\n\tAverage acceptance rate: %.2f%%, ', naccept / ntotal * 100); toc

  for t = 1:T
    % See current stats
    [stat.mse(t), stat.auroc(t), stat.auprc(t)] = eval_accuracy(G_list, D, I_mat, aux_data.true_gbn);
    fprintf('MSE(%.4f), AUROC(%.4f), AUPRC(%.4f), ', stat.mse(t), stat.auroc(t), stat.auprc(t));
    stat.runtime(t) = toc(tstart);
    fprintf('elapsed time(%.0fs)\n', stat.runtime(t));

    tic
    score = zeros(length(C), 1);

    fprintf('[Iter %d] Deciding which experiment to perform ... ', t); tic
    if length(C) > 1
      if strcmp(batch_select_method, 'active')
        % Load required auxiliary parameters
        R = aux_param.R;

        posterior = cell(S, 1);
        for s = 1:S
          posterior{s} = compute_posterior(G_list{s}, D, I_mat, true);
        end

        % Sample x
        gbn_q = learn_MAP_gbn_given_graph(G_singleton, D, I_mat);
        x_all = zeros(R, n);
        logq_all = zeros(R, n);
        for j = 1:n
          param = struct('mu', gbn_q.param.mu(j), 'sigma2', gbn_q.param.sigma2(j), 'W', 0);
          gbn = struct('n', 1, 'adj', 0, 'param', param);
          [x_all(:,j), logq_all(:,j)] = generate_data_from_gbn(gbn, {}, R);
        end

        for i = 1:length(C)
          x = x_all;
          for l = 1:length(C{i})
            x(:,C{i}(l).var) = C{i}(l).val;
          end
          logq = sum(logq_all(:,~I_list2mat(n, C(i))), 2);

          % Compute likelihood
          logp = zeros(R, S);
          parfor s = 1:S
            for r = 1:R
              logp(r,s) = compute_marginal_likelihood_single(G_list{s}, posterior{s}, x(r,:), C{i});
            end
          end

          % Compute score
          weight = bsxfun(@minus, logp, logq);
          weight = bsxfun(@minus, weight, logsumexp(weight, 1));
          log_ratio = log(S) + bsxfun(@minus, logp, logsumexp(logp, 2));
          score(i) = mean(sum(exp(weight) .* log_ratio));
        end
        [~, opt] = max(score);
      elseif strcmp(batch_select_method, 'random')
        opt = randi(length(C));
      else
        error(['Unsupported batch selection method: ', batch_select_method]);
      end
    else
      opt = 1;
    end 

    stat.expr{t} = C{opt};
    fprintf('done.\n\tSelected experiment: %s, ', I2str(C{opt})); toc

    % Perform intervention
    x = generate_data_from_gbn(aux_data.true_gbn, C{opt}, batch_size);
    D = [D; x];
    I_mat = [I_mat; repmat(I_list2mat(n, C(opt)), batch_size, 1)];

    fprintf('Running MCMC on updated data (size = %d) ... ', size(D, 1)); tic
    naccept = zeros(S, 1);
    ntotal = zeros(S, 1);
    parfor s = 1:S
      [G_list(s), G_logp(s), naccept(s), ntotal(s)] = MH_sample_graph(1, D, I_mat, proposal_type, aux_param.MH_update, 0, G_list{s}, false);
    end
    fprintf('done.\n\tAverage acceptance rate: %.2f%%, ', sum(naccept) / sum(ntotal) * 100); toc
  end

  % See final stats
  [stat.mse(end), stat.auroc(end), stat.auprc(end)] = eval_accuracy(G_list, D, I_mat, aux_data.true_gbn);
  fprintf('MSE(%.4f), AUROC(%.4f), AUPRC(%.4f), ', stat.mse(end), stat.auroc(end), stat.auprc(end));
  stat.runtime(end) = toc(tstart);
  fprintf('elapsed time(%.0fs)\n', stat.runtime(end));


  function [mse, auroc, auprc] = eval_accuracy(G_list, D, I_mat, true_gbn)
    selfedge = logical(eye(n));

    W = zeros(n * (n - 1), 1);
    for s = 1:length(G_list)
      gbn = learn_MAP_gbn_given_graph(G_list{s}, D, I_mat, {});
      W = W + gbn.param.W(~selfedge);
    end
    W = W / length(G_list);

    labels = true_gbn.adj(~selfedge);
    mse = mean((true_gbn.param.W(~selfedge) - W) .^ 2);
    [~, ~, ~, auroc] = perfcurve(labels, abs(W), true, 'xCrit', 'FPR', 'yCrit', 'TPR');
    [~, ~, ~, auprc] = perfcurve(labels, abs(W), true, 'xCrit', 'reca', 'yCrit', 'prec');
  end
end
